/* 
 * File:   global.h
 * Author: Jose Antonio Molinet Berenguer
 *
 * Created on 15 de septiembre de 2013, 10:55 PM
 */

# ifndef GLOBAL_H
# define GLOBAL_H

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <time.h>
#include <string.h>
#include <sys/unistd.h>
#include "rand.h"

#ifdef	__cplusplus
extern "C" {
#endif

    /* structures */
    typedef struct
    {
        double *vars;   /* real variables of the problem */
        double *objs;   /* objective functions */
    }
    individual;
    
    extern individual *oldpop; /* population of parents */
    extern individual *newpop; /* population of childs */
    extern double **weights;   /* weights vectors */
    extern double **solutions; /* solutions vectors */
    extern int *aux;           /* auxiliary structure */
    extern int *sel;           /* selection from old and new population */
    
    /* parameters */
    extern int popsize;        /* population size */
    extern int parts;          /* divisions of each dimension of weights vectors */
    extern int generations;    /* number of generations */
    extern int nvar;           /* number of variables */
    extern int nobj;           /* number of objective functions */
    extern double *upBound;    /* upper bound for each variable */
    extern double *lowBound;   /* lower bound for each variable */    
    extern double F;           /* mutation factor in DE */
    extern double CR;          /* crossover factor in DE */
    extern unsigned long seed; /* seed for the random generator */    
    extern void (*functions)(double*,double*,int,int);     /* objective function */
    
    /*** principal functions ***/
    void InitMemory();
    void FreeMemory();
    double rnd(double, double);
    int rndint(int, int);
    void InitialPopulation();
    void GenerateOffspring();
    void Evaluate(individual*);
    void Select();
    void ReadParameters(char*);
    void SaveSolutions(char*);	
	int hde(int, char**);
    
    /* weights vector using Uniform Design of Mixture */
    double RadicalInverse(int, int);
    void Hammersley(double **, int, int);
    void UDM();
    
    /* weights vector using Simplex Lattice Points */
    int NumberOfWeights();
    int AuxWeights(int, int);
    void GetWeights();
    
    /* functions for Kuhn-Munkres algorithm */
    void InitMemoryKM(int, int, int);
    void FreeMemoryKM();
    void Normalize(double **);
    void CostMatrix(double **, double **);
    void InitMunkres(double **, double **);
    void step_one();
    void step_two();
    void step_three();
    void find_a_zero(int *, int *);    
    int find_star_in_row(int );
    void step_four();
    void find_star_in_col(int , int *);
    void find_prime_in_row(int , int *);
    void augment_path();
    void clear_covers();
    void erase_primes();
    void step_five();
    double find_smallest();
    void step_six();
    void step_seven();
    void RunMunkres();
    int* hungarian(double **, double **);





#ifdef	__cplusplus
}
#endif

# endif	/* GLOBAL_H */
